@@ -1,4 +1,35 @@
 
+/* babel.c
+ *
+ * This file is part of fizmo.
+ *
+ * Copyright (c) 2009 Christoph Ender.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+
 #include <fcntl.h>
 #include <string.h>
 #include <unistd.h>
@@ -191,10 +222,10 @@ static int add_doc_to_babel_info(xmlDocPtr new_babel_doc,
 #endif
 
 
+#ifndef DISABLE_LIBXML2
 struct babel_info *load_babel_info_from_blorb(FILE *infile, int length,
     char *filename, struct stat *stat_buf)
 {
-#ifndef DISABLE_LIBXML2
   struct babel_info *result;
   //struct babel_doc_entry *new_entry;
   char *xmlData = (char*)fizmo_malloc(length + 1);
@@ -242,10 +273,14 @@ struct babel_info *load_babel_info_from_blorb(FILE *infile, int length,
   }
 
   return result;
-#else
-  return NULL;
-#endif
 }
+#else
+struct babel_info *load_babel_info_from_blorb(FILE *UNUSED(infile),
+    int UNUSED(length), char *UNUSED(filename), struct stat *UNUSED(stat_buf))
+{
+  return NULL;
+}
+#endif
 
 
 struct babel_info *load_babel_info()
@@ -423,10 +458,10 @@ static char *getStoryNodeContent(xmlXPathContextPtr xpathCtx, char *nodeName,
 #endif // DISABLE_LIBXML2
 
 
+#ifndef DISABLE_LIBXML2
 struct babel_story_info *get_babel_story_info(uint16_t release, char *serial,
     uint16_t checksum, struct babel_info *babel, bool babel_from_blorb)
 {
-#ifndef DISABLE_LIBXML2
   xmlXPathContextPtr xpathCtx; 
   xmlXPathObjectPtr xpathObj;
   char *xmlNamespacedExpr="/if:ifindex/if:story[if:identification/if:ifid='ZCODE-%d-%6s' or translate(if:identification/if:ifid, 'ABCDEF', 'abcdef')=translate('ZCODE-%d-%6s-%04x', 'ABCDEF', 'abcdef')]";
@@ -523,10 +558,15 @@ struct babel_story_info *get_babel_story_info(uint16_t release, char *serial,
   }
 
   return result;
-#else // DISABLE_LIBXML2
-  return NULL;
-#endif // DISABLE_LIBXML2
 }
+#else // DISABLE_LIBXML2
+struct babel_story_info *get_babel_story_info(uint16_t UNUSED(release),
+    char *UNUSED(serial), uint16_t UNUSED(checksum),
+    struct babel_info *UNUSED(babel), bool UNUSED(babel_from_blorb))
+{
+  return NULL;
+}
+#endif // DISABLE_LIBXML2
 
 
 void store_babel_info_timestamps(struct babel_info *babel)
