@@ -1,4 +1,35 @@
 
+/* savegame.c
+ *
+ * This file is part of fizmo.
+ *
+ * Copyright (c) 2009 Christoph Ender.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ * 3. The name of the author may not be used to endorse or promote products
+ *    derived from this software without specific prior written permission.
+ * 
+ * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
+ * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
+ * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+ * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+ * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
+ * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ */
+
+
 #ifndef savegame_c_INCLUDED
 #define savegame_c_INCLUDED
 
@@ -380,7 +411,7 @@ void save_game(uint16_t address, uint16_t length, char *filename,
     bool skip_asking_for_filename, bool evaluate_result, char *directory)
 {
   uint32_t pc_on_restore = (uint32_t)(pc - z_mem);
-  FILE *iff_file;
+  FILE *save_file;
   uint8_t *dynamic_index;
   uint16_t consecutive_zeros;
   int data;
@@ -433,11 +464,12 @@ void save_game(uint16_t address, uint16_t length, char *filename,
 
   TRACE_LOG("Filename to save to: \"%s\".\n", str);
 
-  iff_file = open_simple_iff_file(
-      str,
-      IFF_MODE_WRITE);
+  if (address != 0)
+    save_file = fopen(str, "w");
+  else
+    save_file = open_simple_iff_file(str, IFF_MODE_WRITE);
 
-  if (iff_file == NULL)
+  if (save_file == NULL)
   {
     if (i18n_translate(
           i18n_fizmo_COULD_NOT_OPEN_FILE_NAMED_P0S, str)
@@ -463,308 +495,331 @@ void save_game(uint16_t address, uint16_t length, char *filename,
     return;
   }
 
-  if (directory != NULL)
-    free(str);
-  free(system_filename);
-
-  if (start_new_chunk("IFhd", iff_file) != 0)
+  if (address != 0)
   {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  // Save release number
-  if ((fputc((int)z_mem[0x2], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x3], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  // Save serial number
-  if ((fputc((int)z_mem[0x12], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x13], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x14], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x15], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x16], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x17], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  // Save checksum
-  if ((fputc((int)z_mem[0x1c], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)z_mem[0x1d], iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  // Save initial PC on restore
-  if ((fputc((int)(pc_on_restore >> 16), iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)(pc_on_restore >>  8), iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-  if ((fputc((int)(pc_on_restore      ), iff_file)) == EOF)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  if (end_current_chunk(iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-    return;
-  }
-
-  dynamic_index = z_mem + address;
-
-  if (
-      (active_z_story->z_file != NULL)
-      &&
-      ((fseek(
-              active_z_story->z_file,
-              active_z_story->story_file_exec_offset,
-              SEEK_SET)) == 0)
-      &&
-      (strcmp(get_configuration_value("force-quetzal-umem"), "true") != 0)
-     )
-  {
-    // The original story file is availiable, use CMem.
-    TRACE_LOG("Compressing memory from byte %d ...\n", address);
-
-    if (start_new_chunk("CMem", iff_file) != 0)
+    if ((fwrite(z_mem + address, 1, length, save_file)) != length)
     {
-      _handle_save_or_restore_failure(evaluate_result,
-          i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
+      _handle_save_or_restore_failure(
+          evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
       return;
     }
 
-    ptr = z_mem + address + length;
-    consecutive_zeros = 0;
-
-    while (dynamic_index != ptr)
+    if ((fclose(save_file)) != 0)
     {
-      if ((data = fgetc(active_z_story->z_file)) == EOF)
-      {
-        _handle_save_or_restore_failure(evaluate_result,
-            i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-        return;
-      }
-
-      data2 = data; // removeme
-      data ^= *dynamic_index;
-
-      if (data == 0)
-      {
-        consecutive_zeros++;
-      }
-      else
-      {
-        TRACE_LOG("Altered byte at offset %ld.\n",
-            (long int)(dynamic_index - z_mem));
-
-        TRACE_LOG("Memory-Data: %x, Story-Data: %x.\n",
-            *dynamic_index, data2);
-
-        //TRACE_LOG("Skipping %d equal bytes.\n", consecutive_zeros);
-        while (consecutive_zeros != 0)
-        {
-          if (fputc(0, iff_file) == EOF)
-          {
-            _handle_save_or_restore_failure(evaluate_result,
-                i18n_fizmo_ERROR_WRITING_SAVE_FILE, iff_file);
-            return;
-          }
-
-          if (consecutive_zeros > 256)
-          {
-            if (fputc(0xff, iff_file) == EOF)
-            {
-              _handle_save_or_restore_failure(evaluate_result,
-                  i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-                  iff_file);
-              return;
-            }
-
-            consecutive_zeros -= 256;
-          }
-          else
-          {
-            if (fputc((int)(consecutive_zeros - 1), iff_file) == EOF)
-            {
-              _handle_save_or_restore_failure(evaluate_result,
-                  i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-                  iff_file);
-              return;
-            }
-
-            consecutive_zeros = 0;
-          }
-        }
-
-        if (fputc(data, iff_file) == EOF)
-        {
-          _handle_save_or_restore_failure(evaluate_result,
-              i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-              iff_file);
-          return;
-        }
-      }
-
-      dynamic_index++;
-    }
-    TRACE_LOG("... to byte %ld.\n", (long int)(ptr - z_mem));
-
-    if (end_current_chunk(iff_file) != 0)
-    {
-      _handle_save_or_restore_failure(evaluate_result,
+      _handle_save_or_restore_failure(
+          evaluate_result,
           i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-          iff_file);
+          save_file);
       return;
     }
   }
   else
   {
-    if (start_new_chunk("UMem", iff_file) != 0)
-      i18n_translate_and_exit(
-          i18n_fizmo_FUNCTION_CALL_P0S_ABORTED_DUE_TO_ERROR,
-          -0x0100,
-          "start_new_chunk");
+    if (directory != NULL)
+      free(str);
+    free(system_filename);
 
-    while (dynamic_index != active_z_story->static_memory)
+    if (start_new_chunk("IFhd", save_file) != 0)
     {
-      if ((fputc((int)(*dynamic_index), iff_file)) == EOF)
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    // Save release number
+    if ((fputc((int)z_mem[0x2], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x3], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    // Save serial number
+    if ((fputc((int)z_mem[0x12], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x13], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x14], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x15], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x16], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x17], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    // Save checksum
+    if ((fputc((int)z_mem[0x1c], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)z_mem[0x1d], save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    // Save initial PC on restore
+    if ((fputc((int)(pc_on_restore >> 16), save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)(pc_on_restore >>  8), save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+    if ((fputc((int)(pc_on_restore      ), save_file)) == EOF)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    if (end_current_chunk(save_file) != 0)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+      return;
+    }
+
+    dynamic_index = z_mem + address;
+
+    if (
+        (active_z_story->z_file != NULL)
+        &&
+        ((fseek(
+                active_z_story->z_file,
+                active_z_story->story_file_exec_offset,
+                SEEK_SET)) == 0)
+        &&
+        (strcmp(get_configuration_value("force-quetzal-umem"), "true") != 0)
+       )
+    {
+      // The original story file is availiable, use CMem.
+      TRACE_LOG("Compressing memory from byte %d ...\n", address);
+
+      if (start_new_chunk("CMem", save_file) != 0)
+      {
+        _handle_save_or_restore_failure(evaluate_result,
+            i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+        return;
+      }
+
+      ptr = z_mem + address + length;
+      consecutive_zeros = 0;
+
+      while (dynamic_index != ptr)
+      {
+        if ((data = fgetc(active_z_story->z_file)) == EOF)
+        {
+          _handle_save_or_restore_failure(evaluate_result,
+              i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+          return;
+        }
+
+        data2 = data; // removeme
+        data ^= *dynamic_index;
+
+        if (data == 0)
+        {
+          consecutive_zeros++;
+        }
+        else
+        {
+          TRACE_LOG("Altered byte at offset %ld.\n",
+              (long int)(dynamic_index - z_mem));
+
+          TRACE_LOG("Memory-Data: %x, Story-Data: %x.\n",
+              *dynamic_index, data2);
+
+          //TRACE_LOG("Skipping %d equal bytes.\n", consecutive_zeros);
+          while (consecutive_zeros != 0)
+          {
+            if (fputc(0, save_file) == EOF)
+            {
+              _handle_save_or_restore_failure(evaluate_result,
+                  i18n_fizmo_ERROR_WRITING_SAVE_FILE, save_file);
+              return;
+            }
+
+            if (consecutive_zeros > 256)
+            {
+              if (fputc(0xff, save_file) == EOF)
+              {
+                _handle_save_or_restore_failure(evaluate_result,
+                    i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+                    save_file);
+                return;
+              }
+
+              consecutive_zeros -= 256;
+            }
+            else
+            {
+              if (fputc((int)(consecutive_zeros - 1), save_file) == EOF)
+              {
+                _handle_save_or_restore_failure(evaluate_result,
+                    i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+                    save_file);
+                return;
+              }
+
+              consecutive_zeros = 0;
+            }
+          }
+
+          if (fputc(data, save_file) == EOF)
+          {
+            _handle_save_or_restore_failure(evaluate_result,
+                i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+                save_file);
+            return;
+          }
+        }
+
+        dynamic_index++;
+      }
+      TRACE_LOG("... to byte %ld.\n", (long int)(ptr - z_mem));
+
+      if (end_current_chunk(save_file) != 0)
+      {
+        _handle_save_or_restore_failure(evaluate_result,
+            i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+            save_file);
+        return;
+      }
+    }
+    else
+    {
+      if (start_new_chunk("UMem", save_file) != 0)
+        i18n_translate_and_exit(
+            i18n_fizmo_FUNCTION_CALL_P0S_ABORTED_DUE_TO_ERROR,
+            -0x0100,
+            "start_new_chunk");
+
+      while (dynamic_index != active_z_story->static_memory)
+      {
+        if ((fputc((int)(*dynamic_index), save_file)) == EOF)
+        {
+          _handle_save_or_restore_failure(
+              evaluate_result,
+              i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+              save_file);
+          return;
+        }
+        dynamic_index++;
+      }
+
+      if (end_current_chunk(save_file) != 0)
       {
         _handle_save_or_restore_failure(
             evaluate_result,
             i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-            iff_file);
+            save_file);
         return;
       }
-      dynamic_index++;
     }
 
-    if (end_current_chunk(iff_file) != 0)
+    if (start_new_chunk("Stks", save_file) != 0)
     {
       _handle_save_or_restore_failure(
           evaluate_result,
           i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-          iff_file);
+          save_file);
       return;
     }
-  }
-
-  if (start_new_chunk("Stks", iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(
-        evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
-  }
 
 #ifdef ENABLE_TRACING
-  dump_stack_to_tracelog();
-  dump_dynamic_memory_to_tracelog();
+    dump_stack_to_tracelog();
+    dump_dynamic_memory_to_tracelog();
 #endif // ENABLE_TRACING
 
-  // Save stack frames
-  if (save_stack_frame(
-        z_stack_index-stack_words_from_active_routine-number_of_locals_active,
-        (uint16_t)stack_words_from_active_routine,
-        number_of_locals_active,
-        iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
-  }
+    // Save stack frames
+    if (save_stack_frame(
+          z_stack_index-stack_words_from_active_routine-number_of_locals_active,
+          (uint16_t)stack_words_from_active_routine,
+          number_of_locals_active,
+          save_file) != 0)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
+      return;
+    }
 
-  if (end_current_chunk(iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
-  }
+    if (end_current_chunk(save_file) != 0)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
+      return;
+    }
 
-  if (start_new_chunk("ANNO", iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(
-        evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
-  }
+    if (start_new_chunk("ANNO", save_file) != 0)
+    {
+      _handle_save_or_restore_failure(
+          evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
+      return;
+    }
 
-  fprintf(iff_file, "Interpreter: fizmo, version: %s.\n", FIZMO_VERSION);
+    fprintf(save_file, "Interpreter: fizmo, version: %s.\n", FIZMO_VERSION);
 
-  if (end_current_chunk(iff_file) != 0)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
-  }
+    if (end_current_chunk(save_file) != 0)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
+      return;
+    }
 
-  if (close_simple_iff_file(iff_file) != -0)
-  {
-    _handle_save_or_restore_failure(evaluate_result,
-        i18n_fizmo_ERROR_WRITING_SAVE_FILE,
-        iff_file);
-    return;
+    if (close_simple_iff_file(save_file) != -0)
+    {
+      _handle_save_or_restore_failure(evaluate_result,
+          i18n_fizmo_ERROR_WRITING_SAVE_FILE,
+          save_file);
+      return;
+    }
   }
 
   if (bool_equal(evaluate_result, true))
